import { Injector } from '../../core/injector';
import { OrmMetadataT, OrmMetadata } from './orm';
import { SSLMetadataT, SSLMetadata } from './ssl';
import { RouterMetadataT, RouterMetadata } from './router';
import { ViewsMetadataT, ViewsMetadata } from './views';
import { ProvideMetadata } from './provide';
import { CtorT } from '../../core/ctor';
import { ServiceT } from '../../core/service';
import { JwtMetadataT, JwtMetadata } from './jwt';
export declare type LogLevel = 'verbose' | 'warn' | 'info' | 'error';
export declare type ServerMetadataT = {
    name: string;
    port?: number | string;
    allowCrossOrigin?: boolean;
    ssl?: SSLMetadataT;
    router?: RouterMetadataT | null;
    orm?: OrmMetadataT;
    services?: CtorT<ServiceT>[];
    views?: ViewsMetadataT | null;
    inject?: ProvideMetadata<any>[];
    jwt?: JwtMetadataT | null;
    logLevel?: LogLevel | {
        [name: string]: LogLevel;
    };
    debugBreakpoint?: () => void;
};
export declare class ServerMetadata implements ServerMetadataT {
    private _meta;
    constructor(_meta: ServerMetadataT, injector: Injector);
    readonly name: string;
    readonly port: string | number;
    readonly allowCrossOrigin: boolean;
    private _ssl;
    readonly ssl: SSLMetadata;
    private _router;
    readonly router: RouterMetadata | null;
    private _views;
    readonly views: ViewsMetadata | null;
    private _orm;
    readonly orm: OrmMetadata;
    readonly services: CtorT<ServiceT>[];
    readonly inject: ProvideMetadata<any>[];
    private _jwt;
    readonly jwt: JwtMetadata | null;
    readonly logLevel: "verbose" | "warn" | "info" | "error" | {
        [name: string]: LogLevel;
    };
    readonly debugBreakpoint: () => void;
}
